<?php

/**
 *
 * @link       http://if-so.com
 * @since      1.0.0
 *
 * @package    IfSo
 * @subpackage IfSo/services
 */

/**
 * 
 * Holds settings information
 *
 * @since      1.0.0
 * @package    IfSo
 * @subpackage IfSo/services
 * @author     Matan Green

 */

if (!class_exists('If_So_Plugin_Settings_Service')) {

class If_So_Plugin_Settings_Service {

	const PAGES_VISITED_OPTION =
		'ifso_settings_page_pages_visited_option';
	const REMOVE_PLUGIN_DATA_OPTION =
		'ifso_settings_page_remove_data_on_uninstall_option';
	const APPLY_THE_CONTENT_FILTER_OPTION =
		'ifso_settings_page_apply_the_content_filter_option';
	const REMOVE_AUTO_P_TAG_OPTION =
		'ifso_settings_page_remove_auto_p_tag_option';

	private static $instance;

	public $pagesVisitedOption;
	public $removePluginDataOption;
	public $applyTheContentFilterOption;
	public $removeAutoPTagOption;

	private function __construct() {
		$this->pagesVisitedOption = 
			$this->create_pages_visited_option();
		$this->removePluginDataOption = 
			$this->create_remove_plugin_data_option();
		$this->applyTheContentFilterOption = 
			$this->create_apply_the_content_filter_option();
		$this->removeAutoPTagOption = 
			$this->create_remove_auto_p_tag_option();
	}

	private function create_remove_auto_p_tag_option() {
		$default = false;
		$postName = 'ifso_settings_pages_remove_auto_p_tag';
		$option = new IfSoSettingsYesNoOption(
				self::REMOVE_AUTO_P_TAG_OPTION,
				$default,
				$postName
			);
		return $option;	
	}

	private function create_apply_the_content_filter_option() {
		$default = true;
		$postName = 'ifso_settings_pages_apply_the_content_filter';
		$option = new IfSoSettingsYesNoOption(
				self::APPLY_THE_CONTENT_FILTER_OPTION,
				$default,
				$postName
			);
		return $option;	
	}

	private function create_remove_plugin_data_option() {
		$default = false;
		$postName = 'ifso_settings_pages_remove_data_uninstall';
		$option = new IfSoSettingsYesNoOption(
				self::REMOVE_PLUGIN_DATA_OPTION,
				$default,
				$postName
			);
		return $option;
	}

	private function create_pages_visited_option() {
		$default = array(
			'duration_type' => 'weeks',
			'duration_value' => 2
		);
		$option = new IfSoSettingsPagesVisitedOption(
				self::PAGES_VISITED_OPTION,
				$default
			);
		return $option;
	}

	public static function getInstance() {
		if ( NULL == self::$instance )
			self::$instance = new If_So_Plugin_Settings_Service();

		return self::$instance;
	}

	public function settings_page_update() {
		if( isset( $_POST['ifso_settings_page_update'] ) ) {
		 	if( ! check_admin_referer(
		 			'ifso_settings_nonce',
		 			'ifso_settings_nonce' ) )
				return;

			$this->pagesVisitedOption->apply($_POST);
			$this->removePluginDataOption->apply($_POST);
			$this->applyTheContentFilterOption->apply($_POST);
			$this->removeAutoPTagOption->apply($_POST);

			$this->redirect_to_settings_page();
		}
	}

	private function redirect_to_settings_page() {
		$redirect = admin_url( 'admin.php?page=' . EDD_IFSO_PLUGIN_SETTINGS_PAGE );
		wp_redirect( $redirect );
		exit();
	}
}

abstract class IfSoSettingsOptionBase {
	private $optionName;
	private $optionDefault;

	public function __construct($optionName, $optionDefault) {
		$this->optionName = $optionName;
		$this->optionDefault = $optionDefault;
	}

	public function get() {
		if ( $this->isset_cached_value() )
			return $this->get_cached_value();

		$option = get_option( $this->optionName,
						  	  $this->optionDefault );

		try {
			$this->cache_value($option);
		} catch (InvalidArgumentException $e) {
			$this->cache_value($this->optionDefault);
		}

		return $this->get_cached_value();
	}

	public function set( $optionValue ) {
		update_option( $this->optionName, $optionValue );
		$this->cache_value( $optionValue );
	}

	abstract public function apply($post);
	abstract public function validate($optionValue);
	abstract protected function isset_cached_value();
	abstract protected function get_cached_value();
	abstract protected function cache_value($optionValue);
}

class IfSoSettingsPagesVisitedOption
	extends IfSoSettingsOptionBase {

	private $cachedValue;

	public function apply( $post ) {
		$postName_Type = 'ifso_settings_pages_visited_type';
		$postName_Value = 'ifso_settings_pages_visited_value';

		if ( ! isset( $post[$postName_Type] ) )
			return false;
		else if ( ! isset( $post[$postName_Value] ))
			return false;

		$type = $post[$postName_Type];
		$value = $post[$postName_Value];
		$option = array(
				'duration_type' => $type,
				'duration_value' => $value
			);

		if ( ! $this->validate($option) )
			return false;

		$this->set($option);
	}

	public function validate( $optionValue ) {
		$durationValue = $optionValue['duration_value'];
		if ( ! is_numeric( $durationValue ) )
			return false;

		$durationType = $optionValue['duration_type'];
		$availableDurationTypes = array(
				'minutes',
				'hours',
				'days',
				'weeks',
				'months');
		if ( ! in_array( $durationType,
						 $availableDurationTypes ) )
			return false;

		return true;
	}

	protected function isset_cached_value() {
		return isset( $this->cachedValue );
	}

	protected function get_cached_value() {
		return $this->cachedValue;
	}

	protected function cache_value( $optionValue ) {
		$this->cachedValue = 
			new IfSoPagesVisitedOptionData($optionValue);
	}
}

class IfSoSettingsYesNoOption extends IfSoSettingsOptionBase {

	protected $cachedValue;
	protected $postName;

	public function __construct($optionName,
								$optionDefault,
								$postName) {
		parent::__construct($optionName, $optionDefault);
		$this->postName = $postName;
	}

	public function apply( $post ) {
		$value = isset( $post[$this->postName] ) ? 1 : 0;

		if ( ! $this->validate($value) )
			return false;

		$this->set($value);
	}

	public function validate( $optionValue ) {
		return $optionValue == 1 || $optionValue == 0;
	}

	protected function isset_cached_value() {
		return isset( $this->cachedValue );
	}

	protected function get_cached_value() {
		return $this->cachedValue;
	}

	protected function cache_value( $optionValue ) {
		$this->cachedValue =  $optionValue;
	}
}

class IfSoPagesVisitedOptionData {

	private $durationType;
	private $durationValue;

	public function __construct($options) {
		$this->durationType
			= $this->safe_extrap_option($options, 'duration_type');
		$this->durationValue
			= $this->safe_extrap_option($options, 'duration_value');
	}

	/*
	 * Helper method that extracts the given $option from $arr
	 */
	private function safe_extrap_option($options, $option) {
		if ( key_exists( $option, $options ) )
			return $options[$option];
		else
			throw new InvalidArgumentException($option);
	}

	/*
	 * Returns hours/days/weeks/months
	 */
	public function get_duration_type(){
		return $this->durationType;
	}

	public function get_duration_value(){
		return $this->durationValue;
	}
}

}